/*
 * Copyright 2012-2020 by PaX Team <pageexec@freemail.hu>
 * Licensed under the GPL v2
 *
 * Homepage: http://pax.grsecurity.net/
 */

#include "rap.h"

static bool rap_reload_gate(void)
{
	return rap_cmodel_check();
}

/*
 * find all asm() stmts that clobber rap_reg
 */
static unsigned int rap_reload_execute(void)
{
	basic_block bb;

	// 1. loop through BBs and GIMPLE statements
	FOR_EACH_BB_FN(bb, cfun) {
		gimple_stmt_iterator gsi;

		for (gsi = gsi_start_bb(bb); !gsi_end_p(gsi); gsi_next(&gsi)) {
			// gimple match: asm("" :  :  : "rap_reg_name");
			gasm *asm_stmt;
			size_t nclobbers;

			// is it an asm ...
			if (gimple_code(gsi_stmt(gsi)) != GIMPLE_ASM)
				continue;
			asm_stmt = as_a_gasm(gsi_stmt(gsi));

			// ... clobbering rap_reg_id
			nclobbers = gimple_asm_nclobbers(asm_stmt);
			while (nclobbers--) {
				tree op = gimple_asm_clobber_op(asm_stmt, nclobbers);
				int reg = decode_reg_name(TREE_STRING_POINTER(TREE_VALUE(op)));

				if (reg < 0 || rap_reg_id != (unsigned int)reg)
					continue;
				if (TREE_THIS_VOLATILE(current_function_decl))
					continue;

				error_at(gimple_location(asm_stmt), "asm clobbers reserved RAP register %s", rap_reg_name);
				fflush(stderr);
				print_gimple_stmt(stderr, asm_stmt, 0, TDF_LINENO);
				debug_tree(current_function_decl);
				break;
			}
		}
	}

	return 0;
}

#define PASS_NAME rap_reload
#define TODO_FLAGS_FINISH TODO_verify_ssa | TODO_verify_stmts | TODO_dump_func | TODO_remove_unused_locals | TODO_update_ssa_no_phi
#include "gcc-generate-gimple-pass.h"
