/*
 * Copyright 2012-2020 by PaX Team <pageexec@freemail.hu>
 * Licensed under the GPL v2
 *
 * colorize gcc output as gcc 9 and newer do it
 */

#include "utilities.h"

#if BUILDING_GCC_VERSION < 4009
#define GREEN		"\033[32m\033[K"
#define LIGHTGREEN	"\033[1;32m\033[K"
#define YELLOW		"\033[33m\033[K"
#define LIGHTYELLOW	"\033[1;33m\033[K"
#define RED		"\033[31m\033[K"
#define LIGHTRED	"\033[1;31m\033[K"
#define BLUE		"\033[34m\033[K"
#define LIGHTBLUE	"\033[1;34m\033[K"
#define BRIGHT		"\033[1;m\033[K"
#define NORMAL		"\033[m\033[K"

static diagnostic_starter_fn old_starter;
static diagnostic_finalizer_fn old_finalizer;

static void start_colorize(diagnostic_context *context, diagnostic_info *diagnostic)
{
	const char *color;
	char *newprefix;

	switch (diagnostic->kind) {
	case DK_NOTE:
		color = LIGHTBLUE;
		break;

	case DK_PEDWARN:
	case DK_WARNING:
		color = LIGHTYELLOW;
		break;

	case DK_ERROR:
	case DK_FATAL:
	case DK_ICE:
	case DK_PERMERROR:
	case DK_SORRY:
		color = LIGHTRED;
		break;

	default:
		color = NORMAL;
	}

	old_starter(context, diagnostic);
	if (-1 == asprintf(&newprefix, "%s%s" NORMAL, color, context->printer->prefix))
		return;
	pp_destroy_prefix(context->printer);
	pp_set_prefix(context->printer, newprefix);
}

static void finalize_colorize(diagnostic_context *context, diagnostic_info *diagnostic)
{
	old_finalizer(context, diagnostic);
}

static void colorize_arm(void)
{
	old_starter = diagnostic_starter(global_dc);
	old_finalizer = diagnostic_finalizer(global_dc);

	diagnostic_starter(global_dc) = start_colorize;
	diagnostic_finalizer(global_dc) = finalize_colorize;
}

static unsigned int colorize_rearm_execute(void)
{
	if (diagnostic_starter(global_dc) == start_colorize)
		return 0;

	colorize_arm();
	return 0;
}

#define PASS_NAME colorize_rearm
#define NO_GATE
#include "gcc-generate-simple_ipa-pass.h"

void colorize_start_unit(void *gcc_data __unused, void *user_data __unused)
{
	colorize_arm();
}

bool should_colorize(void)
{
	char const *t = getenv("TERM");

	return t && strcmp(t, "dumb") && isatty(STDERR_FILENO);
}
#endif
